// ClassExampleDlg.cpp : implementation file
//

#include "stdafx.h"
#include "ClassExample.h"
#include "ClassExampleDlg.h"
#include "Td3dDrawable.h"
#include "Td3dZoomDrawable.h"
#include "TtextureControl.h"

// this is the world ...
#include "Model/Tworld.h"

// To use GraphicsAPI functionality
#include "GraphicsAPI/GraphicsAPI.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// under the table mechanism for echo message to the main window Echo Area
CClassExampleDlg *gGlobalDlg = NULL;
void EchoToStatusArea(CString msg)
{
	if (gGlobalDlg != NULL) {
		gGlobalDlg->m_EchoArea = msg;
		gGlobalDlg->UpdateData(false);
	}
}

// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	enum { IDD = IDD_ABOUTBOX };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

// Implementation
protected:
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()


// CClassExampleDlg dialog



CClassExampleDlg::CClassExampleDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CClassExampleDlg::IDD, pParent)
	, m_EchoArea(_T(""))
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	fDrawable = NULL;
	fMyWorld = NULL;
	fTextureControl = NULL;
}


CClassExampleDlg::~CClassExampleDlg()
{
	if (fMyWorld != NULL)
		delete fMyWorld;

	if (fDrawable != NULL)
		delete fDrawable;

	// Release D3D only after the drawables are destroyed
	if (fD3D != NULL)
		fD3D->Release();

	if (fTextureControl != NULL)
		delete fTextureControl;
}


void CClassExampleDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_STATUS_AREA, m_EchoArea);
}

BEGIN_MESSAGE_MAP(CClassExampleDlg, CDialog)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
	ON_WM_TIMER()
END_MESSAGE_MAP()


// CClassExampleDlg message handlers

BOOL CClassExampleDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// support global echoing of message to root window status area
	gGlobalDlg = this;

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	// TODO: Add extra initialization here

	
    // Now create the abstract D3d output pipeline connection
	if( NULL == ( fD3D = Direct3DCreate9( D3D_SDK_VERSION ) ) ) {
		MessageBox("D3d Creation Failed (1)!");
		return FALSE;
	}

	
	fDrawable = new Td3dDrawable();
	LinkDrawableToD3d(IDC_D3DX_PLACEHOLDER, fDrawable);
	fDrawable->SetBgColor(0.7f, 0.8f, 0.8f);
	fDrawable->SetWorldBound(T2dBound(-5, 5, -5, 5));
								//X min/max Y min/max
	

	fZoomDrawable = new Td3dZoomDrawable();\
	LinkDrawableToD3d(IDC_D3DX_PLACEHOLDER_ZOOM, fZoomDrawable);
	fZoomDrawable->SetBgColor(0.6f, 0.6f, 0.9f);
	fZoomDrawable->SetWorldBound(T2dBound(-10, 10, -10, 10));
									// Xmin/max  Y min/max
	fZoomDrawable->SetMainDrawable(fDrawable);

	//
	// Now, create the new world ...
	fMyWorld = new Tworld(*fDrawable, *fZoomDrawable);
	fDrawable->SetWorld(fMyWorld);
	fZoomDrawable->SetWorld(fMyWorld);

	//
	// Now create and show the fire truck control
	fTextureControl = new TtextureControl(this);
	LinkControlToPlaceHolder(IDC_CONTROL_PLACEHOLDER, IDD_TEXTURE_INFO, fTextureControl);
	fTextureControl->SetDrawables(fZoomDrawable, fDrawable);
	fTextureControl->SetWorld(fMyWorld);


	//		Here we initialize the timer, to go off every 25 milli seconds
	//		when timer "goes-off", our OnTimer() function will be called and
	//		it is upto us to decide what we want to do.
	//
	const static int msInterval = 25;	// timer going off every 25 milli seconds.
  	SetTimer(0, msInterval, NULL);		// our goal is to approx 40 frames per second

	CheckRadioButton(IDC_SELECT_POINT, IDC_SELECT_CIRCLE, IDC_SELECT_POINT);

	UpdateData(false);
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CClassExampleDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
	{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
	}
	else
	{
		CDialog::OnSysCommand(nID, lParam);
	}
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CClassExampleDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.

HCURSOR CClassExampleDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CClassExampleDlg::OnBnClickedCancel()
{
	// TODO: Add your control notification handler code here
	OnCancel();
}

void CClassExampleDlg::OnTimer(UINT nIDEvent)
{
	if (fMyWorld != NULL) {
		const T2dBound &r = fZoomDrawable->GetWorldBound();
		fMyWorld->UpdateWorld(*fDrawable, r);
	}

	if (fDrawable != NULL) {
		fDrawable->RedrawDrawable();
		fZoomDrawable->RedrawDrawable();
	}

}