#include "Tcircle.h"
#include "GraphicsAPI.h"
#include "TmyVertex.h"

	const int	Tcircle::sfNumVertices = 128;
	TmyVertex	Tcircle::sfCircleVertices[Tcircle::sfNumVertices+2];

	static Tcircle sfCircleCollisionCircle;

void Tcircle::sInitializeCircle(Td3dDrawable*)
{
	sfCircleVertices[0].fPoint = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
	sfCircleVertices[0].fColor = D3DCOLOR_COLORVALUE(1.0, 1.0, 1.0, 0.0);

		// this is texture coordinate ...
		sfCircleVertices[0].fU = 
		sfCircleVertices[0].fV =
		sfCircleVertices[0].fU1 =
		sfCircleVertices[0].fV1 = 0.5f;

	float delta = (2.0f * M_PI) / float(sfNumVertices);
	for (int i=1; i<=sfNumVertices; i++) {
		// geomtric information of a unit size circle located at the origin.
		sfCircleVertices[i].fPoint.x = sinf((i-1)*delta);
		sfCircleVertices[i].fPoint.y = cosf((i-1)*delta);
		sfCircleVertices[i].fPoint.z = 0.0f;
		sfCircleVertices[i].fColor = D3DCOLOR_COLORVALUE(8.0, 0.6, 0.8, 0.0);

		// Set up texture coordinate ...
		sfCircleVertices[i].fU = sfCircleVertices[i].fU1 = (sfCircleVertices[i].fPoint.x+1.0f) * 0.5f;
		sfCircleVertices[i].fV = sfCircleVertices[i].fV1 = (1.0f - sfCircleVertices[i].fPoint.y) * 0.5f;
	}
	sfCircleVertices[sfNumVertices+1] = sfCircleVertices[1];
}



Tcircle::Tcircle() 
{

	fNumCollisionCircles = 1;
	fCollisionCirclesInOC = &sfCircleCollisionCircle;
}

Tcircle::~Tcircle() 
{
	fNumCollisionCircles = 0;
	fCollisionCirclesInOC = NULL;
	// Cirle cannot have collision circles (recursively defined!)
	// we must let TgraphicsObject destructor know to now try
	// to delete anything ...
}

bool Tcircle::IntersectsBound(const T2dBound &b)
{
	const D3DXVECTOR3 &c = GetCenter();
	float r = GetRadius();
	T2dBound myB(c.x-r, c.x+r, c.y-r, c.y+r);

	return b.BoundIntersect(myB);
}


void Tcircle::EnableShowCollisionCircles()
{
	// since circles themselves are collision circle
	// this is _NEVER_ enabled for Tcircle
	return;
}

void Tcircle::SetSecondPosition(const D3DXVECTOR3 &p)
{
	D3DXVECTOR3 v = p - GetCenter();
	float l = D3DXVec3Length(&v);
	SetRadius(l);
}

float Tcircle::GetRadius() 
{
	return fXformInfo.ScaleX();
}

const D3DXVECTOR3& Tcircle::GetCenter()
{
	return fXformInfo.Movement();
}


void Tcircle::SetRadius(float r) 
{
	fXformInfo.SetScaleTo(r);
}

void Tcircle::SetCenter(const D3DXVECTOR3 &c)
{
	return TgraphicsObject::SetFirstPosition(c);
}

void Tcircle::SetCircle(
	const D3DXVECTOR3 &p,
	float	r)
{
	fXformInfo.SetMoveTo(p);
	fXformInfo.SetScaleTo(r);
}


void Tcircle::DrawObjectGeom(
	Td3dDrawable&,
	LPDIRECT3DDEVICE9 d3dDevice
)
{
	float r = GetRadius();
	if (r <= 0.0f)
		return;

	// set color ...
	sfCircleVertices[0].fColor = D3DCOLOR_COLORVALUE(1.0, 1.0, 1.0, 0.0);
	for (int i=1; i<=(sfNumVertices+1); i++) {
		sfCircleVertices[i].fColor = D3DCOLOR_COLORVALUE(fDrawColor.r, fDrawColor.g, fDrawColor.b, 0.0);
	}

	d3dDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, sfNumVertices, (CONST void *)sfCircleVertices, sizeof(TmyVertex));
}
