#include "TdrawTexture.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
TdrawTexture::~TdrawTexture()
{
	if (fEffectBuffer != NULL) {
		// delete fire buffers
  		delete[] fEffectBuffer; 
		fEffectBuffer = NULL;
	}

}


bool TdrawTexture::CreateD3DTexture(
		int textureSize
)
{
  	if (!fD3dDevice) { 
		EchoToStatusArea("DrawTexture creation failed: D3dDevice is not here!");
		return FALSE;
	}
  	if (textureSize < 2 || textureSize > 512 /* || !IsPowerOf2(textureSize) */) {
   		CString msg;
		msg.Format("DrawTexture size (%d) is bad!", textureSize); 
		EchoToStatusArea(msg);
		return FALSE;
  	}

  	fTextureSize = textureSize;

  	// allocate effect buffers
  	fEffectBuffer = new unsigned char[textureSize*textureSize];

  	// clear the buffers to all 1's
  	memset(fEffectBuffer, 0xff, textureSize*textureSize);

  	// create the texture
  	if (FAILED(D3DXCreateTexture(fD3dDevice, 				// for which device
								fTextureSize, fTextureSize, // width and height
								1,  						// mipmap level (always 1 for now)
								0,							// Usage - not used.
    							D3DFMT_A8R8G8B8, 			// texture data formate
								D3DPOOL_MANAGED, 			// Memory managed by D3D
								&fTexture))) 				// return pointer to the D3D Texture buffer
	{	
    	EchoToStatusArea("TdrawTexture::CreateD3DTexture: can't create texture!");
		return FALSE;
  	}

	return TRUE;
}

void TdrawTexture::DrawCircleInTexture(int px, int py, int radius, unsigned int colorIndex)
{
	int rSq = radius * radius;
	int fx, fy;
	int maxSize = fTextureSize-1;

	//
	// Draw a circle with colorIndex into the buffer (texture)
	//
	for (int x=-radius; x<=radius; x++) {
		for (int y=-radius; y<=radius; y++) {
			float d = float(x*x + y*y);
			float ran = RandomNumber(0.0f, 1.0f);

			if ( (d+ran) < rSq ) {
				fx = x + px;
				fy = y + py;

				fx = (fx<0)?0 : (fx>(maxSize)) ? maxSize : fx;
				fy = (fy<0)?0 : (fy>(maxSize)) ? maxSize : fy;

				int index = ((fy)*fTextureSize)+fx;
				fEffectBuffer[index] = colorIndex;
			}
		}
	}
}

void TdrawTexture::UpdateTexture()
{
	if (fEffectBuffer == NULL)
		return; 

	
  	CopyToTexture(fEffectBuffer);
}

bool TdrawTexture::CopyToTexture(unsigned char *buf)
{
  	HRESULT hr;
  	// lock texture
	D3DLOCKED_RECT lockedrect;
	::ZeroMemory(&lockedrect, sizeof(lockedrect));
	
	if (FAILED(hr = fTexture->LockRect(0, &lockedrect, NULL, 0))) return(false);
	
	// our texture surface is now locked, and we can use the pitch to traverse it.
	unsigned char *pSurfBits = static_cast<unsigned char *>(lockedrect.pBits);
	
	int index=0;
	
	for (int y=0; y<fTextureSize; y++) {
	  	for (int x=0; x < fTextureSize; x++) {
	    	// simply copy the same value into RGBA
	    	pSurfBits[index++] = buf[(y*fTextureSize)+x]; // blue
	    	pSurfBits[index++] = buf[(y*fTextureSize)+x]; // green
	    	pSurfBits[index++] = buf[(y*fTextureSize)+x]; // red
	    	pSurfBits[index++] = 255-buf[(y*fTextureSize)+x]; // alpha
			
	  	}
	  	// next line
	  	index += lockedrect.Pitch - (fTextureSize*4);
	}

	// unlock texture surface
	if (FAILED(hr = fTexture->UnlockRect(0))) return(false);
	return(true);
}