#ifndef _TXFORMINFO
#define _TXFORMINFO

#include "../StdAfx.h"
#include "GraphicsAPI.h"


class TxformInfo {
	private:
		D3DXVECTOR3 fMove;		
		D3DXVECTOR3 fScale;	
		float fRotation;  // in radian ...

	public:
		TxformInfo();

		void ResetTransformation();

		void SetUpD3DXformStack(ID3DXMatrixStack *stack) const;

		// accessing the xform info 
		float MoveX() const;
		float MoveY() const;
		float MoveZ() const;
		const D3DXVECTOR3& Movement() const;

		float ScaleX() const;
		float ScaleY() const;
		float ScaleZ() const;
		const D3DXVECTOR3& Scale() const;

		float RotationInRadian() const;
		float RotationInDegree() const;
		
		//
		// Setting the xform info
		//
		// SetXXTo() - funcitons, _sets_ the values of the xform info
		// UpdateXXBy() - funcitons _modifies_ the xform info values by the input
		//
		//         Movement - adds fMove to the inpit
		//		   Scale    - multiplies fScale to the input
		//         Rotate   - adds fRotation to the input.
		//
		void SetMoveTo(const D3DXVECTOR3 &d);
		void SetMoveTo(float x, float y, float z);
		void UpdateMovementBy(const D3DXVECTOR3 &v);
		void UpdateMovementBy(float x, float y, float z);

		void SetScaleTo(const D3DXVECTOR3 &s);
		void SetScaleXTo(float x);
		void SetScaleYTo(float y);
		void SetScaleZTo(float z);
		void SetScaleTo(float s);
		void SetScaleTo(float x, float y, float z);
		void UpdateScaleBy(float s);
		void UpdateScaleBy(float x, float y, float z);

		void SetRotationToRadian(float theta);
		void SetRotationToDegree(float theta);
		void UpdateRotationByRadian(float theta);
		void UpdateRotationByDegree(float theta);		
};


#endif
