// Td3dDrawable.cpp : implementation file
//

#include "stdafx.h"
#include "ClassExample.h"
#include "Td3dDrawable.h"
#include "TsliderCtrlWithEcho.h"

#include "Model/Tworld.h"

// Td3dDrawable

IMPLEMENT_DYNAMIC(Td3dDrawable, CWnd)
Td3dDrawable::Td3dDrawable() :
	fD3DDevice(NULL),
	fWorldBound(-1, 1, -1, 1)
{
	fWorld = NULL;
}

Td3dDrawable::~Td3dDrawable()
{
}


BEGIN_MESSAGE_MAP(Td3dDrawable, CWnd)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
END_MESSAGE_MAP()


void Td3dDrawable::ComputePositions(
  const CPoint &hwPt, 
		CPoint &dcPt,
		D3DXVECTOR3 &wcPt
)
{
	HardwareToDevice(hwPt, dcPt);
	DeviceToWorldTransform(dcPt, wcPt);
}


// Td3dDrawable message handlers
void Td3dDrawable::OnLButtonDown(UINT nFlags, CPoint point)
{
	// check out the MSDN help page, we can figure out if any of the
	// control/shift key is pressed when left mouse button is pressed
	CPoint devicePt;
	D3DXVECTOR3 worldPt;
	ComputePositions(point, devicePt, worldPt);

	CString msg;
	msg.Format("Object Definition starting from: devicePt(%d %d) WorldPt(%f %f)", 
				devicePt.x, devicePt.y, worldPt.x, worldPt.y);
	EchoToStatusArea(msg);

	fWorld->InitObject(worldPt.x, worldPt.y);
}

void Td3dDrawable::OnLButtonUp(UINT nFlags, CPoint point)
{
	CPoint devicePt;
	D3DXVECTOR3 worldPt;
	ComputePositions(point, devicePt, worldPt);

	CString msg;
	msg.Format("Completing Object at: devicePt(%d %d) WorldPt(%f %f)", 
				devicePt.x, devicePt.y, worldPt.x, worldPt.y);
	EchoToStatusArea(msg);

	fWorld->CompleteObjectAt(worldPt.x, worldPt.y);
}

void Td3dDrawable::OnMouseMove(UINT nFlags, CPoint point)
{
	// check out the MSDN help page, we can figure out if any of the
	// control/shift key is pressed when left mouse button is pressed
	CString msg;
	D3DXVECTOR3 worldPt;
	CPoint devicePt;

	if (nFlags & MK_LBUTTON) {

		ComputePositions(point, devicePt, worldPt);
		msg.Format("Dragging Object To: devicePt(%d %d) WorldPt(%f %f)", 
				devicePt.x, devicePt.y, worldPt.x, worldPt.y);
		EchoToStatusArea(msg);
		fWorld->DrawObjectTo(worldPt.x, worldPt.y);
	}

	if (nFlags & MK_RBUTTON) {
		ComputePositions(point, devicePt, worldPt);
		msg.Format("Dragging Object To: devicePt(%d %d) WorldPt(%f %f)", 
				devicePt.x, devicePt.y, worldPt.x, worldPt.y);
		EchoToStatusArea(msg);
		fWorld->MoveObjectTo(worldPt.x, worldPt.y);
	}
}

void Td3dDrawable::OnRButtonDown(UINT nFlags, CPoint point)
{
	CPoint devicePt;
	D3DXVECTOR3 worldPt;
	ComputePositions(point, devicePt, worldPt);

	CString msg;
	msg.Format("Completing Object at: devicePt(%d %d) WorldPt(%f %f)", 
				devicePt.x, devicePt.y, worldPt.x, worldPt.y);
	EchoToStatusArea(msg);

	fWorld->MoveObjectTo(worldPt.x, worldPt.y);
}

void Td3dDrawable::OnRButtonUp(UINT nFlags, CPoint point)
{
	CPoint devicePt;
	D3DXVECTOR3 worldPt;
	CString msg;
	ComputePositions(point, devicePt, worldPt);
	msg.Format("Dragging Object To: devicePt(%d %d) WorldPt(%f %f)", 
			devicePt.x, devicePt.y, worldPt.x, worldPt.y);
	EchoToStatusArea(msg);
	fWorld->MoveObjectTo(worldPt.x, worldPt.y);
}
