#pragma once
// Td3dDrawable

#include "GraphicsAPI/GraphicsAPI.h"
		// D3D specific incldues are in GraphicsAPI

#include "GraphicsAPI/T2dBound.h"

#include <atlsimpcoll.h>
	// for CSimpleArray

class Tworld;
class Ttexture;

class Td3dDrawable : public CWnd
{
	DECLARE_DYNAMIC(Td3dDrawable)

public:
	Td3dDrawable();
	virtual ~Td3dDrawable();

	BOOL	InitD3D(LPDIRECT3D9);
				// Create and initialize D3D Device (Window)
				// This function is implemented in Td3dDrawable_Init.cpp
			
	void	RedrawDrawable();
				// Causes the redraw of the D3D Device (Window)
				// This function is implemented in Td3dDrawable_Draw.cpp
	
	void	SetBgColor(float r, float g, float b) {
				fBgColor = D3DCOLOR_COLORVALUE(r, g, b, 0.0);
					// the four parameters are R, G, B, and A respectively
					// with 0 represent nothing and 1 represent max.
					// e.g. (0,0,0,X) is black, and (1,1,1,X) is white (X- is don't care)
					// don't worry about the A (alpha-channel) for now, we will learn about this
					// later-on in the quarter, may be in the 9th or 10-th week.
			}

		// Point transformation from hardware coordinate to device coordinate
	void HardwareToDevice(const CPoint &hwPt, CPoint &devicePt) {
				devicePt.x = hwPt.x;
				devicePt.y = fHeight - hwPt.y;
						// the transform is a simply flip of y

	}

		//************************ File texture support ***********************
		//  We can create and access file texture for the D3D pipe via these functions
			int CreateFileTexture(const CString&);
			int ReplaceFileTextureAt(int index, const CString &);
			int ReplaceTextureAt(int index, Ttexture *t);
			Ttexture*	GetTexture(int i) { return ( (i>=0) && (i < fTextureLibrary.GetSize())) ? fTextureLibrary[i] : NULL; }
		//**********************************************************************
	
	void SetDrawableSize(int w, int h) { 
			fWidth = w; fHeight = h; 
			SetUpWorldToNDCTransform();
	}
	// accessing method for drawable dimension
	float Width() const { return float(fWidth); }
	float Height() const { return float(fHeight); }

	LPDIRECT3DDEVICE9 D3DDevice() { return fD3DDevice; }
	ID3DXMatrixStack* Obj2WldStack() { return fObj2WorldStack; }

	void SetWorldBound(const T2dBound &rec) {
		fWorldBound = rec;
		SetUpWorldToNDCTransform();
	}
	const T2dBound& GetWorldBound() const { return fWorldBound; }

		// Sets the world
	void SetWorld(Tworld *w) {	fWorld = w;	}

	// Event Service Routines
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);


protected:
	DECLARE_MESSAGE_MAP()

	
	LPDIRECT3DDEVICE9 fD3DDevice;
				// This is the abstract link to the D3D Device (Window)

	
	virtual void DrawableSpecificDraws();
				// This is for subclasses to override if it has anything special to draw

	void ComputePositions(const CPoint &hwPt, CPoint &dcPt, D3DXVECTOR3 &wcPt);

	void SetUpWorldToNDCTransform();
	void DeviceToWorldTransform(const CPoint &devicePt, D3DXVECTOR3 &worldPt);
		float fDevice2WorldWidthScale;
		float fDevice2WorldHeightScale; 
				// these two scale factors for for Device to World transform
				// these two values are computed at the end of SetUpWorldToNDCTransform()
				// these two values are used in DeviceToWorldTransform


	//************************ File texture support ***********************
		void InitializeFileTextureLibrary();
		CSimpleArray<Ttexture*> fTextureLibrary;
	//**********************************************************************
		

	ID3DXMatrixStack	*fObj2WorldStack;	
				// We will use this stack to push/pop matrix to support object
				// to world coordinate transformations

	D3DCOLOR fBgColor;
				// This is the background color

	int		fWidth, fHeight;
				// Height and Width of the Drawable window

	T2dBound	fWorldBound;
				// User's world bound
	T2dBound  fUserRequestedBound;
				// This is what user requested. although sometimes we cannot honor this, we must
				// remember this and always use this as reference for computing new WorldBound.

	Tworld	*fWorld;	// this is the world we are seeing
};