// Td3dZoomDrawable.cpp : implementation file
//
#include "stdafx.h"
#include "ClassExample.h"
#include "Td3dZoomDrawable.h"

#include "Model/Tworld.h"

// Td3dZoomDrawable

IMPLEMENT_DYNAMIC(Td3dZoomDrawable, CWnd)
Td3dZoomDrawable::Td3dZoomDrawable() :
	fMainDrawable(NULL)
{
	fMainArea.SetShadedMode(D3DSHADE_FLAT);
	fMainArea.SetFillMode(D3DFILL_WIREFRAME);
	fMainArea.SetDrawColor(D3DXCOLOR(0, 1, 0, 0));
}

Td3dZoomDrawable::~Td3dZoomDrawable()
{
}


BEGIN_MESSAGE_MAP(Td3dZoomDrawable, CWnd)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
END_MESSAGE_MAP()


void Td3dZoomDrawable::ComputeMainArea()
{
	const T2dBound &rect = fMainDrawable->GetWorldBound();

	const D3DXVECTOR3 f = rect.MinPosition();
	const D3DXVECTOR3 s = rect.MaxPosition();

	// remember first position is the center ...
	D3DXVECTOR3 c = (f + s) / 2.0f;

	fMainArea.SetFirstPosition(c);
	fMainArea.SetSecondPosition(s);
}


void Td3dZoomDrawable::MoveMainAreaTo(const D3DXVECTOR3 &wcPt)
{
	CString msg;
	msg.Format("Moving Work Drawable Center to WC(%f %f)", wcPt.x, wcPt.y);
	EchoToStatusArea(msg);

	const T2dBound &rect = fMainDrawable->GetWorldBound();
	float w = rect.Width()/2.0f;
	float h = rect.Height()/2.0f;

	fMainDrawable->SetWorldBound(T2dBound(wcPt.x-w, wcPt.x+w, wcPt.y-h, wcPt.y+h));
									// xMin/Max, yMin/Max
}

void Td3dZoomDrawable::ScaleMainDrawableWC(
	float xMove
) {
	// 1. Get the center position of the window ...
	const D3DXVECTOR3 f = fTmpBound.MinPosition();
	const D3DXVECTOR3 s = fTmpBound.MaxPosition();
	D3DXVECTOR3 c = (f + s) / 2.0f;
	
	// 2. Get the scaling value from the valuator to compute the new size
	float scale = fValuator.GetValue(xMove);
	float w = 0.5f * fTmpBound.Width() * scale;
	float h = 0.5f * fTmpBound.Height() * scale;

	// 3. Set the new size
	T2dBound newB(c.x-w, c.x+w, c.y-h, c.y+h);
	fMainDrawable->SetWorldBound(newB);

	CString msg;
	msg.Format("Scaline World Coordinate of Work Drawable by Scale(%f) With Respect To(%f %f)", scale, c.x, c.y);
	EchoToStatusArea(msg);
}

// Td3dZoomDrawable message handlers

static const float kScaleRange = 1.0f;	// scale up to twice of the original size, but no more
static const float kMaxScale = 1.0f + kScaleRange;
static const float kMinScale = (kScaleRange * 0.1f);
								// we will only allow shrinking to 90% of scaling range
								// or, scale-factor is 10% of the scaling range
static const float kXRightMovement = 50.0f;
static const float kXLeftMovement = (0.9f*kXRightMovement);
								// since only supports shrinking of upto 90% of max range

void Td3dZoomDrawable::OnLButtonDown(UINT nFlags, CPoint point)
{
	CPoint dcPt;
	D3DXVECTOR3 wcPt;
	ComputePositions(point, dcPt, wcPt);

	fValuator.InitializeValuator(point.x-kXLeftMovement, point.x+kXRightMovement, kMinScale, kMaxScale);
	fTmpBound = fMainDrawable->GetWorldBound();
}

void Td3dZoomDrawable::OnLButtonUp(UINT nFlags, CPoint point)
{
	CPoint dcPt;
	D3DXVECTOR3 wcPt;
	ComputePositions(point, dcPt, wcPt);
	ScaleMainDrawableWC(point.x);
		
}

void Td3dZoomDrawable::OnMouseMove(UINT nFlags, CPoint point)
{
	CPoint dcPt;
	D3DXVECTOR3 wcPt;
	ComputePositions(point, dcPt, wcPt);

	if (nFlags & MK_LBUTTON) {
		// Scale the work window ...
		ScaleMainDrawableWC(point.x);
	}

	if (nFlags & MK_RBUTTON) {
		MoveMainAreaTo(wcPt);
	}

}

void Td3dZoomDrawable::OnRButtonDown(UINT nFlags, CPoint point)
{
	CPoint dcPt;
	D3DXVECTOR3 wcPt;
	ComputePositions(point, dcPt, wcPt);

	MoveMainAreaTo(wcPt);
	
}

void Td3dZoomDrawable::OnRButtonUp(UINT nFlags, CPoint point)
{
	CPoint dcPt;
	D3DXVECTOR3 wcPt;
	ComputePositions(point, dcPt, wcPt);
	
	MoveMainAreaTo(wcPt);
}